# coding: utf-8

from __future__ import absolute_import
from __future__ import unicode_literals

from datetime import datetime


class SimpleField(object):

    """
    Дескриптор для удобного описания и получения данных из словаря data 
    объекта класса.
    Поддерживает вложенные структуры и доступ к ним осуществляется через точку.
    
    :param path: Путь до значения. например, person.contacts.phone
    :type path: str or unicode
    
    :param default: Значение по умолчанию, может быть callable объектом.
    :param choices: Используется для выбора значения из словаря (mapping).
    :type choices: dict or tuple of tuple or list of list
    """

    def __init__(self, path, default=None, choices=None):
        self.choices = dict(choices) if choices else None
        self.default = default
        self.path = path

    def __get__(self, instance, owner):
        default = self.default() if callable(self.default) else self.default
        keys = self.path.split('.')
        result = instance.data.get(keys.pop(0))
        for key in keys:
            result = result.get(key) if result else None

        if self.choices:
            result = self.choices.get(result)

        return result if result is not None else default


class DateField(SimpleField):

    """
    Дескриптор для удобного описания и получения даты из словаря data 
    объекта класса.
    Поддерживает вложенные структуры и доступ к ним осуществляется через точку.
    
    :param path: Путь до значения. например, person.date_of_birth
    :type path: str or unicode
    
    :param d_format: Формат даты в виде строки. Например, %d-%m-%Y.
    :type d_format: str or unicode
    :param default: Значение по умолчанию, может быть callable объектом.
    """

    def __init__(self, path, d_format, default=None):
        super(DateField, self).__init__(path, default)
        self.d_format = d_format

    def __get__(self, instance, owner):
        result = super(DateField, self).__get__(instance, owner)
        if result is not None:
            result = datetime.strptime(result, self.d_format).date()
        return result
